"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");
exports.__esModule = true;
exports.default = void 0;
var _stampit = _interopRequireDefault(require("stampit"));
var _visitor = require("../traversal/visitor.cjs");
var _index = _interopRequireDefault(require("./value/index.cjs"));
const YamlVisitor = (0, _stampit.default)({
  props: {
    result: '',
    indent: 0,
    indentChar: '  '
  },
  init({
    signature = '',
    indent = 0,
    indentChar = '  '
  } = {}) {
    this.result = signature;
    this.indent = indent;
    this.indentChard = indentChar;
  },
  methods: {
    NumberElement(element) {
      this.result += (0, _index.default)(element);
    },
    BooleanElement(element) {
      const value = (0, _index.default)(element);
      this.result += value ? 'true' : 'false';
    },
    StringElement(element) {
      // for simplicity and avoiding ambiguity we always wrap strings in quotes
      this.result += JSON.stringify((0, _index.default)(element));
    },
    NullElement() {
      this.result += 'null';
    },
    ArrayElement(element) {
      if (element.length === 0) {
        this.result += '[]';
        return false;
      }
      element.forEach(item => {
        const visitor = YamlVisitor({
          indent: this.indent + 1
        });
        const indent = this.indentChar.repeat(this.indent);
        (0, _visitor.visit)(item, visitor);
        const {
          result
        } = visitor;
        this.result += result.startsWith('\n') ? `\n${indent}-${visitor.result}` : `\n${indent}- ${visitor.result}`;
      });
      return false;
    },
    ObjectElement(element) {
      if (element.length === 0) {
        this.result += '{}';
        return false;
      }
      element.forEach((value, key) => {
        const keyVisitor = YamlVisitor({
          indent: this.indent + 1
        });
        const valueVisitor = YamlVisitor({
          indent: this.indent + 1
        });
        const indent = this.indentChar.repeat(this.indent);
        (0, _visitor.visit)(key, keyVisitor);
        (0, _visitor.visit)(value, valueVisitor);
        const {
          result: keyResult
        } = keyVisitor;
        const {
          result: valueResult
        } = valueVisitor;
        this.result += valueResult.startsWith('\n') ? `\n${indent}${keyResult}:${valueResult}` : `\n${indent}${keyResult}: ${valueResult}`;
      });
      return false;
    }
  }
});
const serializer = element => {
  const signature = '%YAML 1.2\n---\n';
  const visitor = YamlVisitor({
    signature
  });
  (0, _visitor.visit)(element, visitor);
  return visitor.result;
};
var _default = serializer;
exports.default = _default;