import stampit from 'stampit';
import { ArrayElement, BREAK } from '@swagger-api/apidom-core';
import SpecificationVisitor from "../SpecificationVisitor.js";
import FallbackVisitor from "../FallbackVisitor.js";
import ParentSchemaAwareVisitor from "./ParentSchemaAwareVisitor.js";
import { isJSONReferenceLikeElement } from "../../predicates.js";
const ItemsVisitor = stampit(SpecificationVisitor, ParentSchemaAwareVisitor, FallbackVisitor, {
  methods: {
    ObjectElement(objectElement) {
      const specPath = isJSONReferenceLikeElement(objectElement) ? ['document', 'objects', 'JSONReference'] : ['document', 'objects', 'JSONSchema'];
      this.element = this.toRefractedElement(specPath, objectElement);
      return BREAK;
    },
    ArrayElement(arrayElement) {
      this.element = new ArrayElement();
      this.element.classes.push('json-schema-items');
      arrayElement.forEach(item => {
        const specPath = isJSONReferenceLikeElement(item) ? ['document', 'objects', 'JSONReference'] : ['document', 'objects', 'JSONSchema'];
        const element = this.toRefractedElement(specPath, item);
        this.element.push(element);
      });
      this.copyMetaAndAttributes(arrayElement, this.element);
      return BREAK;
    }
  }
});
export default ItemsVisitor;